@@ -5,12 +5,14 @@ import logging
 
 from homeassistant.components import (
     button,
+    climate,
     cover,
     fan,
     image_processing,
     input_button,
     input_number,
     light,
+    media_player,
     timer,
     vacuum,
 )
@@ -18,8 +20,6 @@ from homeassistant.components.alarm_control_panel import (
     AlarmControlPanelEntityFeature,
     CodeFormat,
 )
-import homeassistant.components.climate.const as climate
-import homeassistant.components.media_player.const as media_player
 from homeassistant.const import (
     ATTR_CODE_FORMAT,
     ATTR_SUPPORTED_FEATURES,
@@ -269,6 +269,7 @@ class Alexa(AlexaCapability):
     """
 
     supported_locales = {
+        "ar-SA",
         "de-DE",
         "en-AU",
         "en-CA",
@@ -277,10 +278,13 @@ class Alexa(AlexaCapability):
         "en-US",
         "es-ES",
         "es-MX",
+        "es-US",
         "fr-CA",
         "fr-FR",
+        "hi-IN",
         "it-IT",
         "ja-JP",
+        "pt-BR",
     }
 
     def name(self):
@@ -295,6 +299,7 @@ class AlexaEndpointHealth(AlexaCapability):
     """
 
     supported_locales = {
+        "ar-SA",
         "de-DE",
         "en-AU",
         "en-CA",
@@ -302,9 +307,14 @@ class AlexaEndpointHealth(AlexaCapability):
         "en-IN",
         "en-US",
         "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
         "fr-FR",
+        "hi-IN",
         "it-IT",
         "ja-JP",
+        "pt-BR",
     }
 
     def __init__(self, hass, entity):
@@ -345,6 +355,7 @@ class AlexaPowerController(AlexaCapability):
     """
 
     supported_locales = {
+        "ar-SA",
         "de-DE",
         "en-AU",
         "en-CA",
@@ -352,9 +363,14 @@ class AlexaPowerController(AlexaCapability):
         "en-IN",
         "en-US",
         "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
         "fr-FR",
+        "hi-IN",
         "it-IT",
         "ja-JP",
+        "pt-BR",
     }
 
     def name(self):
@@ -379,7 +395,7 @@ class AlexaPowerController(AlexaCapability):
             raise UnsupportedProperty(name)
 
         if self.entity.domain == climate.DOMAIN:
-            is_on = self.entity.state != climate.HVAC_MODE_OFF
+            is_on = self.entity.state != climate.HVACMode.OFF
         elif self.entity.domain == fan.DOMAIN:
             is_on = self.entity.state == fan.STATE_ON
         elif self.entity.domain == vacuum.DOMAIN:
@@ -400,6 +416,7 @@ class AlexaLockController(AlexaCapability):
     """
 
     supported_locales = {
+        "ar-SA",
         "de-DE",
         "en-AU",
         "en-CA",
@@ -461,9 +478,14 @@ class AlexaSceneController(AlexaCapability):
         "en-IN",
         "en-US",
         "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
         "fr-FR",
+        "hi-IN",
         "it-IT",
         "ja-JP",
+        "pt-BR",
     }
 
     def __init__(self, entity, supports_deactivation):
@@ -483,6 +505,7 @@ class AlexaBrightnessController(AlexaCapability):
     """
 
     supported_locales = {
+        "ar-SA",
         "de-DE",
         "en-AU",
         "en-CA",
@@ -490,6 +513,9 @@ class AlexaBrightnessController(AlexaCapability):
         "en-IN",
         "en-US",
         "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
         "fr-FR",
         "hi-IN",
         "it-IT",
@@ -536,6 +562,9 @@ class AlexaColorController(AlexaCapability):
         "en-IN",
         "en-US",
         "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
         "fr-FR",
         "hi-IN",
         "it-IT",
@@ -587,6 +616,9 @@ class AlexaColorTemperatureController(AlexaCapability):
         "en-IN",
         "en-US",
         "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
         "fr-FR",
         "hi-IN",
         "it-IT",
@@ -635,9 +667,13 @@ class AlexaPercentageController(AlexaCapability):
         "en-IN",
         "en-US",
         "es-ES",
+        "es-US",
+        "fr-CA",
         "fr-FR",
+        "hi-IN",
         "it-IT",
         "ja-JP",
+        "pt-BR",
     }
 
     def name(self):
@@ -758,7 +794,24 @@ class AlexaPlaybackController(AlexaCapability):
     https://developer.amazon.com/docs/device-apis/alexa-playbackcontroller.html
     """
 
-    supported_locales = {"de-DE", "en-AU", "en-CA", "en-GB", "en-IN", "en-US", "fr-FR"}
+    supported_locales = {
+        "ar-SA",
+        "de-DE",
+        "en-AU",
+        "en-CA",
+        "en-GB",
+        "en-IN",
+        "en-US",
+        "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
+        "fr-FR",
+        "hi-IN",
+        "it-IT",
+        "ja-JP",
+        "pt-BR",
+    }
 
     def name(self):
         """Return the Alexa API name of this interface."""
@@ -792,7 +845,24 @@ class AlexaInputController(AlexaCapability):
     https://developer.amazon.com/docs/device-apis/alexa-inputcontroller.html
     """
 
-    supported_locales = {"de-DE", "en-AU", "en-CA", "en-GB", "en-IN", "en-US"}
+    supported_locales = {
+        "ar-SA",
+        "de-DE",
+        "en-AU",
+        "en-CA",
+        "en-GB",
+        "en-IN",
+        "en-US",
+        "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
+        "fr-FR",
+        "hi-IN",
+        "it-IT",
+        "ja-JP",
+        "pt-BR",
+    }
 
     def name(self):
         """Return the Alexa API name of this interface."""
@@ -830,6 +900,7 @@ class AlexaTemperatureSensor(AlexaCapability):
     """
 
     supported_locales = {
+        "ar-SA",
         "de-DE",
         "en-AU",
         "en-CA",
@@ -837,9 +908,14 @@ class AlexaTemperatureSensor(AlexaCapability):
         "en-IN",
         "en-US",
         "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
         "fr-FR",
+        "hi-IN",
         "it-IT",
         "ja-JP",
+        "pt-BR",
     }
 
     def __init__(self, hass, entity):
@@ -901,12 +977,18 @@ class AlexaContactSensor(AlexaCapability):
         "de-DE",
         "en-AU",
         "en-CA",
+        "en-GB",
         "en-IN",
         "en-US",
-        "en-GB",
         "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
+        "fr-FR",
+        "hi-IN",
         "it-IT",
         "ja-JP",
+        "pt-BR",
     }
 
     def __init__(self, hass, entity):
@@ -950,10 +1032,15 @@ class AlexaMotionSensor(AlexaCapability):
         "de-DE",
         "en-AU",
         "en-CA",
+        "en-GB",
         "en-IN",
         "en-US",
-        "en-GB",
         "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
+        "fr-FR",
+        "hi-IN",
         "it-IT",
         "ja-JP",
         "pt-BR",
@@ -997,6 +1084,7 @@ class AlexaThermostatController(AlexaCapability):
     """
 
     supported_locales = {
+        "ar-SA",
         "de-DE",
         "en-AU",
         "en-CA",
@@ -1004,7 +1092,11 @@ class AlexaThermostatController(AlexaCapability):
         "en-IN",
         "en-US",
         "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
         "fr-FR",
+        "hi-IN",
         "it-IT",
         "ja-JP",
         "pt-BR",
@@ -1127,6 +1219,8 @@ class AlexaPowerLevelController(AlexaCapability):
         "en-IN",
         "en-US",
         "es-ES",
+        "es-MX",
+        "fr-CA",
         "fr-FR",
         "it-IT",
         "ja-JP",
@@ -1260,10 +1354,13 @@ class AlexaModeController(AlexaCapability):
         "en-US",
         "es-ES",
         "es-MX",
+        "es-US",
         "fr-CA",
         "fr-FR",
+        "hi-IN",
         "it-IT",
         "ja-JP",
+        "pt-BR",
     }
 
     def __init__(self, entity, instance, non_controllable=False):
@@ -1446,10 +1543,13 @@ class AlexaRangeController(AlexaCapability):
         "en-US",
         "es-ES",
         "es-MX",
+        "es-US",
         "fr-CA",
         "fr-FR",
+        "hi-IN",
         "it-IT",
         "ja-JP",
+        "pt-BR",
     }
 
     def __init__(self, entity, instance, non_controllable=False):
@@ -1691,8 +1791,10 @@ class AlexaToggleController(AlexaCapability):
         "en-US",
         "es-ES",
         "es-MX",
+        "es-US",
         "fr-CA",
         "fr-FR",
+        "hi-IN",
         "it-IT",
         "ja-JP",
         "pt-BR",
@@ -1753,6 +1855,7 @@ class AlexaChannelController(AlexaCapability):
     """
 
     supported_locales = {
+        "ar-SA",
         "de-DE",
         "en-AU",
         "en-CA",
@@ -1761,6 +1864,8 @@ class AlexaChannelController(AlexaCapability):
         "en-US",
         "es-ES",
         "es-MX",
+        "es-US",
+        "fr-CA",
         "fr-FR",
         "hi-IN",
         "it-IT",
@@ -1780,6 +1885,7 @@ class AlexaDoorbellEventSource(AlexaCapability):
     """
 
     supported_locales = {
+        "ar-SA",
         "de-DE",
         "en-AU",
         "en-CA",
@@ -1794,6 +1900,7 @@ class AlexaDoorbellEventSource(AlexaCapability):
         "hi-IN",
         "it-IT",
         "ja-JP",
+        "pt-BR",
     }
 
     def name(self):
@@ -1811,7 +1918,24 @@ class AlexaPlaybackStateReporter(AlexaCapability):
     https://developer.amazon.com/docs/device-apis/alexa-playbackstatereporter.html
     """
 
-    supported_locales = {"de-DE", "en-GB", "en-US", "es-MX", "fr-FR"}
+    supported_locales = {
+        "ar-SA",
+        "de-DE",
+        "en-AU",
+        "en-CA",
+        "en-GB",
+        "en-IN",
+        "en-US",
+        "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
+        "fr-FR",
+        "hi-IN",
+        "it-IT",
+        "ja-JP",
+        "pt-BR",
+    }
 
     def name(self):
         """Return the Alexa API name of this interface."""
@@ -1849,7 +1973,24 @@ class AlexaSeekController(AlexaCapability):
     https://developer.amazon.com/docs/device-apis/alexa-seekcontroller.html
     """
 
-    supported_locales = {"de-DE", "en-GB", "en-US", "es-MX"}
+    supported_locales = {
+        "ar-SA",
+        "de-DE",
+        "en-AU",
+        "en-CA",
+        "en-GB",
+        "en-IN",
+        "en-US",
+        "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
+        "fr-FR",
+        "hi-IN",
+        "it-IT",
+        "ja-JP",
+        "pt-BR",
+    }
 
     def name(self):
         """Return the Alexa API name of this interface."""
@@ -1925,7 +2066,24 @@ class AlexaEqualizerController(AlexaCapability):
     https://developer.amazon.com/en-US/docs/alexa/device-apis/alexa-equalizercontroller.html
     """
 
-    supported_locales = {"de-DE", "en-IN", "en-US", "es-ES", "it-IT", "ja-JP", "pt-BR"}
+    supported_locales = {
+        "de-DE",
+        "en-AU",
+        "en-CA",
+        "en-GB",
+        "en-IN",
+        "en-US",
+        "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
+        "fr-FR",
+        "hi-IN",
+        "it-IT",
+        "ja-JP",
+        "pt-BR",
+    }
+
     VALID_SOUND_MODES = {
         "MOVIE",
         "MUSIC",
@@ -2017,6 +2175,7 @@ class AlexaCameraStreamController(AlexaCapability):
     """
 
     supported_locales = {
+        "ar-SA",
         "de-DE",
         "en-AU",
         "en-CA",
@@ -2024,6 +2183,9 @@ class AlexaCameraStreamController(AlexaCapability):
         "en-IN",
         "en-US",
         "es-ES",
+        "es-MX",
+        "es-US",
+        "fr-CA",
         "fr-FR",
         "hi-IN",
         "it-IT",
